@@ -7,14 +7,24 @@ from matplotlib.colors import LogNorm
 from sklearn import metrics
 from sklearn.calibration import CalibratedClassifierCV
 
+from .preprocessing import horizontal_to_camera
 
-def plot_regressor_confusion(performace_df, log_xy=True, log_z=True, ax=None, label_column='label', prediction_column='label_prediction'):
+
+def plot_regressor_confusion(
+        performance_df,
+        log_xy=True,
+        log_z=True,
+        ax=None,
+        label_column='label',
+        prediction_column='label_prediction',
+        energy_unit='GeV'
+):
 
     ax = ax or plt.gca()
 
-    label = performace_df[label_column].copy()
+    label = performance_df[label_column].copy()
 
-    prediction = performace_df[prediction_column].copy()
+    prediction = performance_df[prediction_column].copy()
 
     if log_xy is True:
         label = np.log10(label)
@@ -37,17 +47,32 @@ def plot_regressor_confusion(performace_df, log_xy=True, log_z=True, ax=None, la
     ax.figure.colorbar(img, ax=ax)
 
     if log_xy is True:
-        ax.set_xlabel(r'$\log_{10}(E_{\mathrm{MC}} \,\, / \,\, \mathrm{GeV})$')
-        ax.set_ylabel(r'$\log_{10}(E_{\mathrm{Est}} \,\, / \,\, \mathrm{GeV})$')
+        ax.set_xlabel(
+                rf'$\log_{10}(E_{{\mathrm{{MC}}}} \,\, / \,\, \mathrm{{{energy_unit}}})$'
+        )
+        ax.set_ylabel(
+                rf'$\log_{10}(E_{{\mathrm{{Est}}}} \,\, / \,\, \mathrm{{{energy_unit}}})$'
+        )
     else:
-        ax.set_xlabel(r'$E_{\mathrm{MC}} \,\, / \,\, \mathrm{GeV}$')
-        ax.set_ylabel(r'$E_{\mathrm{Est}} \,\, / \,\, \mathrm{GeV}$')
+        ax.set_xlabel(
+                rf'$E_{{\mathrm{{MC}}}} \,\, / \,\, \mathrm{{{energy_unit}}}$'
+        )
+        ax.set_ylabel(
+                rf'$E_{{\mathrm{{Est}}}} \,\, / \,\, \mathrm{{{energy_unit}}}$'
+        )
 
     return ax
 
 
-def plot_bias_resolution(performace_df, bins=10, ax=None, label_column='label', prediction_column='label_prediction'):
-    df = performace_df.copy()
+def plot_bias_resolution(
+        performance_df,
+        bins=10,
+        ax=None,
+        label_column='label',
+        prediction_column='label_prediction',
+        energy_unit='GeV'
+):
+    df = performance_df.copy()
 
     ax = ax or plt.gca()
 
@@ -88,12 +113,20 @@ def plot_bias_resolution(performace_df, bins=10, ax=None, label_column='label',
         )
     ax.legend()
     ax.set_xscale('log')
-    ax.set_xlabel(r'$\log_{10}(E_{\mathrm{true}} \,\, / \,\, \mathrm{GeV})$')
+    ax.set_xlabel(
+            rf'$\log_{10}(E_{{\mathrm{{MC}}}} \,\, / \,\, \mathrm{{{energy_unit}}})$'
+    )
 
     return ax
 
 
-def plot_roc(performace_df, model, ax=None, label_column='label', score_column='probabilities'):
+def plot_roc(
+        performance_df,
+        model,
+        ax=None,
+        label_column='label',
+        score_column='scores',
+):
 
     ax = ax or plt.gca()
 
@@ -104,8 +137,11 @@ def plot_roc(performace_df, model, ax=None, label_column='label', score_column='
 
     roc_aucs = []
 
-    mean_fpr, mean_tpr, _ = metrics.roc_curve(performace_df[label_column], performace_df[score_column])
-    for it, df in performace_df.groupby('cv_fold'):
+    mean_fpr, mean_tpr, _ = metrics.roc_curve(
+        performance_df[label_column],
+        performance_df[score_column],
+    )
+    for it, df in performance_df.groupby('cv_fold'):
 
         fpr, tpr, _ = metrics.roc_curve(df[label_column], df[score_column])
 
@@ -132,7 +168,15 @@ def plot_roc(performace_df, model, ax=None, label_column='label', score_column='
     return ax
 
 
-def plot_probabilities(performace_df, model, ax=None, xlabel='score', classnames={0:'Proton', 1:'Gamma'}, label_column='label', score_column='probabilities'):
+def plot_scores(
+    performance_df,
+    model,
+    ax=None,
+    xlabel='score',
+    classnames={0: 'Proton', 1: 'Gamma'},
+    label_column='label',
+    score_column='score',
+):
 
     ax = ax or plt.gca()
 
@@ -140,10 +184,12 @@ def plot_probabilities(performace_df, model, ax=None, xlabel='score', classnames
         model = model.base_estimator
 
     n_bins = (model.n_estimators + 1) if hasattr(model, 'n_estimators') else 100
-    bin_edges = np.linspace(performace_df[score_column].min(), performace_df[score_column].max(), n_bins + 1)
-
-
-    for label, df in performace_df.groupby(label_column):
+    bin_edges = np.linspace(
+        performance_df[score_column].min(),
+        performance_df[score_column].max(),
+        n_bins + 1,
+    )
+    for label, df in performance_df.groupby(label_column):
         ax.hist(
             df[score_column],
             bins=bin_edges, label=classnames[label], histtype='step',
@@ -154,7 +200,7 @@ def plot_probabilities(performace_df, model, ax=None, xlabel='score', classnames
     ax.figure.tight_layout()
 
 
-def plot_precision_recall(performace_df, model, ax=None, beta=0.1):
+def plot_precision_recall(performance_df, model, score_column='score', ax=None, beta=0.1):
 
     ax = ax or plt.gca()
 
@@ -171,10 +217,11 @@ def plot_precision_recall(performace_df, model, ax=None, beta=0.1):
     ax.axvline(1, color='lightgray')
     ax.axhline(0, color='lightgray')
     ax.axhline(1, color='lightgray')
+
     for threshold in thresholds:
 
-        prediction = (performace_df.probabilities.values >= threshold).astype('int')
-        label = performace_df.label.values
+        prediction = (performance_df[score_column] >= threshold).astype('int')
+        label = performance_df.label.values
 
         precision.append(metrics.precision_score(label, prediction))
         recall.append(metrics.recall_score(label, prediction))
@@ -244,26 +291,7 @@ def plot_feature_importances(model, feature_names, ax=None, max_features=20):
 def plot_true_delta_delta(data_df, model_config, ax=None):
 
     df = data_df.copy()
-
-    if model_config.coordinate_transformation == 'CTA':
-        from .cta_helpers import horizontal_to_camera_cta_simtel
-        source_x, source_y = horizontal_to_camera_cta_simtel(
-            az=df[model_config.source_az_column],
-            zd=df[model_config.source_zd_column],
-            az_pointing=df[model_config.pointing_az_column],
-            zd_pointing=df[model_config.pointing_zd_column],
-            focal_length=df[model_config.focal_length_column],
-        )
-        df[model_config.delta_column] = np.deg2rad(df[model_config.delta_column])
-    elif model_config.coordinate_transformation == 'FACT':
-        from fact.coordinates.utils import horizontal_to_camera
-        source_x, source_y = horizontal_to_camera(
-            az=df[model_config.source_az_column],
-            zd=df[model_config.source_zd_column],
-            az_pointing=df[model_config.pointing_az_column],
-            zd_pointing=df[model_config.pointing_zd_column],
-        )
-
+    source_x, source_y = horizontal_to_camera(df, model_config)
     true_delta = np.arctan2(
         source_y - df[model_config.cog_y_column],
         source_x - df[model_config.cog_x_column],
@@ -273,3 +301,59 @@ def plot_true_delta_delta(data_df, model_config, ax=None):
     ax.figure.tight_layout()
     ax.set_xlabel(r'$\delta_{true}\,-\,\delta$')
     return ax
+
+
+def plot_energy_dependent_disp_metrics(df, true_energy_column, energy_unit='GeV', fig=None):
+
+    df = df.copy()
+    edges = 10**np.arange(
+        np.log10(df[true_energy_column].min()),
+        np.log10(df[true_energy_column].max()),
+        0.2
+    )
+    df['bin_idx'] = np.digitize(df[true_energy_column], edges)
+
+    def accuracy(group):
+        return metrics.accuracy_score(
+            group.sign,
+            group.sign_prediction,
+        )
+
+    def r2(group):
+        return metrics.r2_score(
+            np.abs(group.disp),
+            group.disp_prediction,
+        )
+
+    # discard under and overflow
+    df = df[(df['bin_idx'] != 0) & (df['bin_idx'] != len(edges))]
+
+    binned = pd.DataFrame({
+        'e_center': 0.5 * (edges[1:] + edges[:-1]),
+        'e_low': edges[:-1],
+        'e_high': edges[1:],
+        'e_width': np.diff(edges),
+    }, index=pd.Series(np.arange(1, len(edges)), name='bin_idx'))
+
+    binned['accuracy'] = df.groupby('bin_idx').apply(accuracy)
+    binned['r2_score'] = df.groupby('bin_idx').apply(r2)
+
+    fig = fig or plt.figure()
+
+    ax1 = fig.add_subplot(2, 1, 1)
+    ax2 = fig.add_subplot(2, 1, 2, sharex=ax1)
+
+    ax1.errorbar(
+        binned.e_center, binned.accuracy, xerr=binned.e_width / 2, ls='',
+    )
+    ax1.set_ylabel(r'Accuracy for $\mathrm{sgn} \mathtt{disp}$')
+
+    ax2.errorbar(binned.e_center, binned.r2_score, xerr=binned.e_width / 2, ls='')
+    ax2.set_ylabel(r'$r^2$ score for $|\mathtt{disp}|$')
+
+    ax2.set_xlabel(
+        r'$E_{\mathrm{true}} \,\,/\,\,' + rf' \mathrm{{{energy_unit}}}$'
+    )
+    ax2.set_xscale('log')
+
+    return fig
